package com.vctgo.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.vctgo.common.mybatisplus.mapper.BaseMapperX;
import com.vctgo.system.api.domain.SysDept;
import com.vctgo.system.api.domain.dto.SysDeptDto;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * 部门管理 数据层
 *
 * @author vctgo
 */
@Repository
 public interface SysDeptMapper extends BaseMapperX<SysDept>
{
    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
     List<SysDept> selectDeptList(SysDept dept);

    /**
     * 查询部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
    List<SysDept> selectDeptTreeData(SysDept dept);

    /**
     * 查询各部门所管理的子部门
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
    List<SysDeptDto> selectManageSubDepartment(SysDept dept);

    /**
     * 根据部门id统计当前部门以及其下所有子部门的用户数
     *
     * @param deptId 部门信息
     * @return 部门信息集合
     */
    Long staticsUserCountByDeptId(Long deptId);

    /**
     * 根据租户id统计当前租户的用户数
     *
     * @param deptId 部门信息
     * @return 部门信息集合
     */
    Long staticsUserCountByTenantId(Long deptId);

    /**
     * 查询关联租户的部门管理数据
     *
     * @param dept 部门信息
     * @return 部门信息集合
     */
     List<SysDept> selectAssociateTenantDeptList(SysDept dept);

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId 角色ID
     * @param deptCheckStrictly 部门树选择项是否关联显示
     * @return 选中部门列表
     */
     List<Long> selectDeptListByRoleId(@Param("roleId") Long roleId, @Param("deptCheckStrictly") boolean deptCheckStrictly);

    /**
     * 根据部门ID查询信息
     *
     * @param deptId 部门ID
     * @return 部门信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysDept selectDeptById(Long deptId);

    /**
     * 根据租户ID查询该租户的顶级部门信息
     *
     * @param tenantId 租户ID
     * @return 部门信息
     */
     @InterceptorIgnore(tenantLine = "1")
     SysDept selectTopDeptByTenantId(Long tenantId);

    /**
     * 根据ID查询所有子部门
     *
     * @param deptId 部门ID
     * @return 部门列表
     */
     List<SysDept> selectChildrenDeptById(Long deptId);

    /**
     * 根据ID查询所有子部门（正常状态）
     *
     * @param deptId 部门ID
     * @return 子部门数
     */
     int selectNormalChildrenDeptById(Long deptId);

    /**
     * 是否存在子节点
     *
     * @param deptId 部门ID
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int hasChildByDeptId(Long deptId);

    /**
     * 查询部门是否存在用户
     *
     * @param deptId 部门ID
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int checkDeptExistUser(Long deptId);

    /**
     * 校验部门名称是否唯一
     *
     * @param deptName 部门名称
     * @param parentId 父部门ID
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     SysDept checkDeptNameUnique(@Param("deptName") String deptName,
                                 @Param("parentId") Long parentId,
                                 @Param("tenantId") Long tenantId);

    /**
     * 新增部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
     int insertDept(SysDept dept);

    /**
     * 修改部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int updateDept(SysDept dept);

     /**
     * 修改部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int manageLeader(SysDept dept);

     /**
     * 修改部门信息
     *
     * @param dept 部门信息
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int updateTenantName(SysDept dept);

    /**
     * 修改所在部门正常状态
     *
     * @param deptIds 部门ID组
     */
     void updateDeptStatusNormal(Long[] deptIds);

    /**
     * 通过用户id清除主管信息
     *
     * @param userId 用户id
     */
    @InterceptorIgnore(tenantLine = "1")
    int clearLeaderInfoByUserId(Long userId);

    /**
     * 修改子元素关系
     *
     * @param depts 子元素
     * @return 结果
     */
     int updateDeptChildren(@Param("depts") List<SysDept> depts);

    /**
     * 删除部门管理信息
     *
     * @param deptId 部门ID
     * @return 结果
     */
     @InterceptorIgnore(tenantLine = "1")
     int deleteDeptById(Long deptId);

    /**
     * 删除部门信息-根据租户ID
     *
     * @param ids 租户id
     * @return 结果
     */
    @InterceptorIgnore(tenantLine = "1")
    int deleteDeptByTenantId(Long[] ids);

    List<SysDept> selectUserByDept(@Param("tenantId") String tenantId,
                                   @Param("nickName") String nickName,
                                   @Param("phoneNumber") String phoneNumber,
                                   @Param("userId") Long userId);
}
